//NON COLlinear AntiFerromagnet
//resonance frequencies
//compiled with DEV C++ 5.11 as Win32 Console App
//(c)V.Glazkov, June 2016
//glazkov@kapitza.ras.ru
//kapitza.ras.ru/people/glazkov

#define _version_ID "v.1.00, June 28, 2016"

#include<stdlib.h>
#include<stdio.h>
#include<math.h>



//Linking Numerical Recipies
#define ANSI
#define NDIM 3 //minimization routine space dimension
#define FTOL 1e-8  // minimization routine tolerance
#include"nrutil.h"
#include"nr.h"

//3d vector algebra 
#include"vector.h"

//defining global variables
//(Energy parameters, Magnetic field etc...)
vector3d dirH;
double a[4][4][4][4],I[4],gamma,Hstart,Hstop,Hstep,euler0[4];
double Hval;
int is_a[4][4][4][4],minsearch_flag,minsearch_grid;

//output filenames
#define _staticfilename "noncolaf.st" //static
#define _freqsfilename "noncolaf.frq"  //AFMR frequencies
#define _eigenvecfilename "noncolaf.mag" //Magnetization components
#define _descriptionfilename "noncolaf.dsk" //Description (model parameters etc...)


//some subroutines
#include"ini.h"//ini-file processing
#include"magvect.h"//antiferromagnetic vectors algebra
#include"energy.h" //energy calculations
#include"minimize.h"//looking for minimum
#include"saving.h"  //saving results


int main(int argc, char *argv[])
{
int ini_errcode;
int iter;
int i;
float fret,*p;
double omega[4];
//command line control
if(argc>2)//too many parameters
	{
	printf("Too many command line parameters specified.\nCorrect options are:\n \t -v\t\tget version information\n");
	return(-2);
	}
if(argc==2)
	{
	if(strnicmp(argv[1],"-v",10)==0)//version info
		{
		printf("NONCOLAF program version: ");	
		printf(_version_ID);
		return(-1);
		}
	printf("Unknown command line key.\nCorrect options are:\n \t -v\t\tget version information\n");
	return(-2);
	}
//reaing ini-file
ini_errcode=ini_read();
if(ini_errcode)
   {
   return(ini_errcode);
   }
//check for consistency
ini_errcode=ini_validate();
if(ini_errcode)
   {
   return(ini_errcode);
   }

p=vector(1,NDIM);//Numerical Recipies object for minimum search

create_headers(_descriptionfilename,_staticfilename,_freqsfilename,_eigenvecfilename);

//field scan loop
for(Hval=Hstart;Hval*Hstep<=Hstop*Hstep;Hval+=Hstep)
	{
	if(minsearch_flag)
		minsearch(euler0);

	for(i=1;i<=3;i++)
		p[i]=(float)euler0[i];

	printf("H=%.2f",(float)Hval);

	//call for minimization routine
	frprmn(p,NDIM,FTOL,&iter,&fret,_U,_grad_U);
	for(i=1;i<=3;i++)
		euler0[i]=(double)p[i];
	printf(".");
	//calculate and save static properties at equilibrium
	save_static(_staticfilename,euler0[1],euler0[2],euler0[3]);
	printf(".");

	//calculate and save oscillations eigenfrequencies
	save_eigenfreqs(_freqsfilename,euler0[1],euler0[2],euler0[3],omega);
	printf(".");

	//calculate oscillations eigenvectors
	save_eigenvecs(_eigenvecfilename,euler0[1],euler0[2],euler0[3],omega);
	printf(".\n");
	}

return(0);
}
